function [U, V, X, C, S, Q] = my_gsvd(A, B)

m = size(A, 1);
p = size(B, 1);

AB = [A; B];
[P, R, Q] = svd(AB);
k = rank(R);
R = R(1 : k, 1 : k);

P11 = P(1 : m, 1 : k);
[U, C, W] = svd(P11);

P21 = P((m + 1) : (m + p), 1 : k);

[V, S] = ql(P21 * W);

X = W' * R;

% Note that under the assumption that I = S'S + C'C, it holds that
%
% S_{ii}^2 / C_{ii}^2 < S_{jj}^2 / C_{jj}^2
% <=>
% S_{ii}^2 C_{jj}^2 < S_{jj}^2 C_{ii}^2 
% <=>
% (1 - C{ii}^2) C_{jj}^2 < (1 - C{jj}^2) C_{ii}^2 
% <=>
% C_{jj}^2 - C{ii}^2 C_{jj}^2 < C_{ii}^2 - C{jj}^2 C_{ii}^2 
% <=>
% C_{jj}^2 < C_{ii}^2
%
% Therefore, if C'C decreases from top to bottom, then
% the _squared_ generalized singular values increase from 
% top to bottom. Since the svd() function does just this, 
% we do not need further reordering to achieve this.
%
% If in addition both S and C are non-negative, then also the
% generalized singular values are in increasing order:
%
% S_{ii} / C_{ii} < S_{jj} / C_{jj}
% <=>
% S_{ii}^2 / C_{ii}^2 < S_{jj}^2 / C_{jj}^2
% <=>
% C_{jj}^2 < C_{ii}^2
% <=>
% C_{jj} < C_{ii}.
%
% The svd() produces a non-negative C. However, S may not
% be non-negative, as computed by the QL-decomposition.

% Change S non-negative.

% S
% d = k - p;
% f = max(d, 0);
% g = max(-d, 0);
% d
% f
% g
% V
% negativeSet = find(diag(S, d) < 0);
% negativeSet
% V(:, g + negativeSet) = -V(:, g + negativeSet);
% S(:, f + negativeSet) = -S(:, f + negativeSet);
% S(S <= 0) = 0;
% S

    
    

